import * as PIXI from 'pixi.js';
// import dragAndDrop from '../../../helpers/dragAndDrop';
import GuiableContainer from '../../../helpers/Guiable';
import Injector from '../../services/Injector';

import UserService from '../../services/UserService';

import Text from '../misc/Text';
import Button from '../misc/Button';

import SocketService from '../../services/SocketService';
import GameService from '../../services/GameService';

import ChatBox from './ChatBox';

import MenuHolder from '../base/Menu';
// const BlueURL = '/files/assets/cards/frame_blue.png';
// const BlueImage = PIXI.Texture.fromImage(BlueURL);

class Menu extends GuiableContainer{
    constructor(props) {
        super(props);
        let {
            x,
            y
        } = props;

        // Properties Component 
        //this.imageURL = image || getParam('imageURL');
        this.position.set(x,y);

        this.options = {
            x: x || 0,
            y: y || 0
        };


        this.chat = new ChatBox();
        this.chat.mount();

        // GUI
        this.addFolder('Menu');
        this.addToFolder('Menu', this.options, 'x').onFinishChange((v) => this.position.x = v);
        this.addToFolder('Menu', this.options, 'y').onFinishChange((v) => this.position.y = v);
        //
        this.construct(props);
    }

    construct(props) {
        let { GameLayer } = props;

        let vertical = new PIXI.Sprite(PIXI.Texture.fromImage('/files/assets/ui/menu/vertical_menu.png'));
        vertical.position.set(-960, -475);
        vertical.scale.set(1);
        this.addChild(vertical);


        var TopBar = new PIXI.Graphics();
        TopBar.beginFill(0x53494e, 1);
        TopBar.drawRect(-960, -540, 1920, 60);
        TopBar.endFill();
        TopBar.alpha = 1;
        this.addChild(TopBar);

        var Selection = new PIXI.Graphics();
        Selection.beginFill(0x53494e, 1);
        Selection.drawRect(0, 0, 400, 80);
        Selection.endFill();
        Selection.alpha = 0;
        this.addChild(Selection);

        let MenuBar = new MenuHolder({GameLayer, elemHeight: 80});
        MenuBar.position.set(-890, -150);
        this.addToMenu = (e) => {
            e.hitArea = new PIXI.Rectangle(-60, -40, 400, 80);
            e.on('mouseover', (() => {
                Selection.alpha = 1;
                let positionLocal = this.parent.toLocal(e.getGlobalPosition());
                Selection.position.set(positionLocal.x - 60, positionLocal.y - 40);
            }));
            e.on('mouseout', (() => {
                Selection.alpha = 0;
            }));
            MenuBar.addToMenu(e);
        }
        this.addChild(MenuBar);

        let info = new Text({ style: 'Small', fontSize:10, align: 'center', x: -745, y: 300, text: '^^'});
        this.addChild(info);
        this.setInfo = (text) => {
            info.setText(text);
        }

        let verticalTop = new PIXI.Sprite(PIXI.Texture.fromImage('/files/assets/ui/menu/vertical_menu_top.png'));
        verticalTop.position.set(-960, -480);
        verticalTop.scale.set(1);
        this.addChild(verticalTop);
    }

    update = () => {
        this.textSprite.setText(UserService.getUsername() + ':' + UserService.getToken());
    }

    _kill() {
        super._kill();
    }

    getAsJSON() {
        return {
            component: 'buildings/Menu',
            x:  this.position.x,
            y:  this.position.y,
            id: this.options.id,
            team: this.options.team
        };
    }
}

export default Menu;