import * as PIXI from 'pixi.js';
// import dragAndDrop from '../../../helpers/dragAndDrop';
import GuiableContainer from '../../../helpers/Guiable';
import Injector from '../../services/Injector';

import UserService from '../../services/UserService';

import Text from '../misc/Text';
import Button from '../misc/Button';

import SocketService from '../../services/SocketService';
import GameService from '../../services/GameService';

import ChatBox from '../buildings/ChatBox';

import MenuBar from '../buildings/Menu';

import AudioService from '../../services/AudioService';

// const BlueURL = '/files/assets/cards/frame_blue.png';
// const BlueImage = PIXI.Texture.fromImage(BlueURL);

class Menu extends GuiableContainer{
    constructor(props) {
        super(props);
        let {
            x,
            y
        } = props;

        // Properties Component 
        //this.imageURL = image || getParam('imageURL');
        this.position.set(x,y);

        this.options = {
            x: x || 0,
            y: y || 0
        };


        this.chat = new ChatBox();
        this.chat.mount();

        // GUI
        this.addFolder('Menu');
        this.addToFolder('Menu', this.options, 'x').onFinishChange((v) => this.position.x = v);
        this.addToFolder('Menu', this.options, 'y').onFinishChange((v) => this.position.y = v);
        //
        this.construct(props);
        AudioService.play(2, {
            start   : 48,
            end     : 85,
            loop    : true,
            speed   : 1
        });
        this._random_queue_size = 0;
    }

    construct(props) {
        let { GameLayer } = props;

        this.parentLayer = Injector.getByName('MainLayer');
        this.textSprite = new Text({text: "123", y: -515});
        this.textSprite.scale.set(0.7);
        this.textSprite.setText(UserService.getUsername() + ':' + UserService.getToken());

        SocketService.openSocket('');
        
        SocketService.to().on('gameInProgress', (match) => {
            SocketService.setGame('randomFree');
            Injector.getByName('Navigator').goToGame();
            GameService.init(match);
        });

        let play = new Button({ y:-100 , Text: {text: "Play Game", align: 'left'}});
        play.onClick((e) => {
            console.log("Play");
            //SocketService.openSocket('randomFree');
            SocketService.to().emit('join_RandomFreeQueue');
            SocketService.setGame('randomFree');
            SocketService.to().once('joinGame', (game) => {
                console.log('Joining Game...');
                console.log(game);
                Injector.getByName('Navigator').goToGame();
                GameService.init(game);
            });
        });


        let collection = new Button({  y:50 , Text: {text: "Collection", align: 'left'}});
        collection.onClick((e) => alert(e));
        let logout = new Button({  y:200 , Text: {text: "Logout", align: 'left'}});
        logout.onClick((e) => {
            SocketService.end();
            let nav = Injector.getByName('Navigator');
            UserService.logout();
            nav.go('Login');
        });


        let menuBackground = new PIXI.Sprite(PIXI.Texture.fromImage('/files/assets/ui/menu/menu_background.png'));
        menuBackground.position.set(-960, -540);
        menuBackground.scale.set(1);
        this.addChild(menuBackground);


        let Sidebar = new MenuBar({GameLayer});
        this.addChild(Sidebar);

        Sidebar.addToMenu(play);
        Sidebar.addToMenu(collection);
        Sidebar.addToMenu(logout);
        this.addChild(this.textSprite);


        play.on('mouseover', () => Sidebar.setInfo("Play with a random\ndeck\nPlayers Searching: " + this._random_queue_size));
        play.on('mouseout', () => Sidebar.setInfo("Players on lobby: " + this._lobby_size));
        collection.on('mouseover', () => Sidebar.setInfo("Browse your cards\nand\ncreate your deck"));
        collection.on('mouseout', () => Sidebar.setInfo("Players on lobby: " + this._lobby_size));


        SocketService.to().on('RandomFreeQueueSize', (size) => {
            console.log(size);
            this._random_queue_size = size;
        });
        SocketService.to().on('lobbySize', (size) => {
            console.log(size);
            this._lobby_size        = size;
            Sidebar.setInfo("Players on lobby: " + this._lobby_size);
        });

        //this.addChild(Online);
        // this.addChild(Lobby);

    }

    update = () => {
        this.textSprite.setText(UserService.getUsername() + ':' + UserService.getToken());
    }

    _kill() {
        super._kill();
    }

    getAsJSON() {
        return {
            component: 'buildings/Menu',
            x:  this.position.x,
            y:  this.position.y,
            id: this.options.id,
            team: this.options.team
        };
    }
}

export default Menu;