import * as PIXI from 'pixi.js';
import Text from './Text.js';
import { getParam } from '../../../helpers/url';

import GuiableContainer from '../../../helpers/Guiable';



// const DefaultImageUrl = '/files/assets/ui/woodenbutton.png';
// const DefaultImage = PIXI.Texture.fromImage(DefaultImageUrl);

class Button extends GuiableContainer{
    constructor(props) {
        super(props);
        let {
            image,
            x,
            y,
            width,
            height
        } = props;

        // Properties Component 
        this.imageURL = image || getParam('imageURL') || '';
        this.position.set(x,y);

        this.x = x || 0;
        this.y = y || 0;

        this.width  = width  || 0;
        this.height = height || 0;

        // GUI
        this.addFolder('Button');
        this.addToFolder('Button', this, 'imageURL').onFinishChange((v) => this.loadImage(v));
        this.addToFolder('Button', this, 'x').onFinishChange((v) => this.position.x = v);
        this.addToFolder('Button', this, 'y').onFinishChange((v) => this.position.y = v);
        // 


        this.construct(props);
    }

    construct(props) {
        this.sprite = new PIXI.Sprite(this.imageURL);
        this.sprite.anchor.set(0.5,0.5);
        // this.sprite.interactive = true;
        // this.sprite.buttonMode = true;
        this.addChild(this.sprite);
    
        //draggable(this.sprite);
        this.textNode = new Text({...(props.Text|| {}), Gui: props.Gui});
        this.addChild(this.textNode);

        this.loadImage(this.imageURL);        

        this.interactive = true;
        this.buttonMode = true;

        this.setText = this.textNode.setText; // Pass setText;
    }

    scaleTo(x) {
        this.scale.set(x);
        return this;
    }

    loadImage(img) {
        this.imageURL = img;
        this.setTexture(PIXI.Texture.fromImage(img));
    }

    setTexture(texture) {
        this.sprite.texture = texture;
    }

    onClick(fn) {
        this.on('pointerdown', (e) => fn(e));
        //this.sprite.on('mouseup', (e) => fn(e));
        return this;
    }


    _kill() {
        this.textNode._kill();
        super._kill();
    }

    getAsJSON() {
        return {
            component: 'misc/Button',
            Text: this.textNode.getAsJSON(),
            image: this.imageURL,
            x: this.position.x,
            y: this.position.y
        };
    }
}

export default Button;