import * as PIXI from 'pixi.js'
import GuiableContainer from '../../../helpers/Guiable'

class Text extends GuiableContainer {
	constructor(props) {
		super(props);
		let {
			text,
			style,
			x,
			y
		} = props;

		this.text = text || '';
		this.style = style || 'Normal';
		this.x = x || 0;
		this.y = y || 0;


		this.addFolder("Text");
		this.addToFolder('Text', this, 'text').onFinishChange((v) => this.setText(v));
		this.addToFolder('Text', this, 'style', TextStylesNames).onFinishChange((v) => this.setStyle(v));
		this.addToFolder('Text', this, 'x').onFinishChange((v) => this.position.x = v);
		this.addToFolder('Text', this, 'y').onFinishChange((v) => this.position.y = v);


		this.construct(props);
	}

	onClick(fn) {
		this.textNode.interactive = true;
		this.textNode.click = fn;
	}

	construct(props) { 
		let { align } = props;
		
		this.textNode = new PIXI.Text(this.text,TextStyles[this.style]);
		switch(align) {
			case 'center':
			default:
				this.textNode.anchor.set(0.5,0.5);
				break;
			case 'left':
				this.textNode.anchor.set(0, 0.5);
				break;
		}
		this.addChild(this.textNode);
	}

	setText = (args) => {
		this.text = args;
		this.textNode.text = args;
	}
	setStyle = (args) => {
		this.style = args;
		this.textNode.style = TextStyles[args];
	}

	getAsJSON = () => {
		return {
			component: 'misc/Text',
			text: this.text,
			style: this.style,
		}
	}
}

let TextStylesNames = {
	Normal: 'Normal',
	Light:  'Light',
	Heavy:  'Heavy',
	Comic:  'Comic',
	Info:   'Info',
	Card:   'Card'
};

let TextStyles = {
	Small: new PIXI.TextStyle({fontFamily : 'MainSolid', fontSize: 16, fill : 0xb2b6ac, align : 'center'}),
	Normal: new PIXI.TextStyle({fontFamily : 'MainSolid', fontSize: 16, fill : 0xb2b6ac, align : 'left'}),
	Light:  new PIXI.TextStyle({fontFamily : 'Arial', fontSize: 22, fill : 0x000000, align : 'center'}),
	Heavy:  new PIXI.TextStyle({fontFamily : 'Tahoma', fontSize: 25, fill : 0x022005, align : 'center'}),
	Comic:  new PIXI.TextStyle({fontFamily : 'Arial', fontSize: 28, fill : 0x000000, align : 'left'}),
	Info: 	new PIXI.TextStyle({fontFamily : 'Arial', fontSize: 28, fill : 0x000000, align : 'center'}),
	Card:   new PIXI.TextStyle({fontFamily : 'Arial', fontSize: 17, fill : 0xffffff, align : 'center'})
};

const calculateStyles = (style) => {
	style.Small.fontSize = 36;
	style.Normal.fontSize = 72;
}
setTimeout(() => {
	calculateStyles(TextStyles);
}, 500);

export {
	TextStyles,
	TextStylesNames
};
export default Text;
